'  declare globals for passing thrust values to 7 engines
reDim thrust(7)
reDim ISP(7)
reDim MassFlowRate(7)

Dim docking_Update_Inputs
Dim docking_Update_Outputs

Dim FuzzyDockingControl
Dim serr 
Dim ScenPath 
Dim EngineIndex

'==========================================================================
' docking_Update() fctn
'==========================================================================
Function docking_Update( argArray )

	Dim retVal

	If IsEmpty(argArray(0)) Then

		' do compute

		retVal = docking_Update_compute( argArray )

	ElseIf argArray(0) = "register" Then

		docking_Update_globalVar = -1

		retVal = docking_Update_register()

	ElseIf argArray(0) = "compute" Then

		' do compute

		retVal = docking_Update_compute( argArray )

	Else

		' bad call

		retVal = Empty

	End If


	docking_Update = retVal

End Function

Function docking_Update_register()

	ReDim descripStr(3), argStr(9)

	descripStr(0)="ArgumentType = Output"
	descripStr(1)="Name = Status"
	descripStr(2)="ArgumentName = status"

	argStr(0) = descripStr

	ReDim descripStr(4)

	descripStr(0)="ArgumentType = Input"
	descripStr(1)="Name = InTrack"
	descripStr(2)="ArgumentName = X"
	descripStr(3)="Type = CalcObject"

	argStr(1) = descripStr

	ReDim descripStr(4)

	descripStr(0)="ArgumentType = Input"
	descripStr(1)="Name = CrossTrack"
	descripStr(2)="ArgumentName = Y"
	descripStr(3)="Type = CalcObject"

	argStr(2) = descripStr

	ReDim descripStr(4)

	descripStr(0)="ArgumentType = Input"
	descripStr(1)="Name = Radial"
	descripStr(2)="ArgumentName = Z"
	descripStr(3)="Type = CalcObject"

	argStr(3) = descripStr

	ReDim descripStr(4)

	descripStr(0)="ArgumentType = Input"
	descripStr(1)="Name = InTrackRate"
	descripStr(2)="ArgumentName = Xdot"
	descripStr(3)="Type = CalcObject"

	argStr(4) = descripStr

	ReDim descripStr(4)

	descripStr(0)="ArgumentType = Input"
	descripStr(1)="Name = CrossTrackRate"
	descripStr(2)="ArgumentName = Ydot"
	descripStr(3)="Type = CalcObject"

	argStr(5) = descripStr

	ReDim descripStr(4)

	descripStr(0)="ArgumentType = Input"
	descripStr(1)="Name = RadialRate"
	descripStr(2)="ArgumentName = Zdot"
	descripStr(3)="Type = CalcObject"

	argStr(6) = descripStr

	ReDim descripStr(4)

	descripStr(0)="ArgumentType = Input"
	descripStr(1)="Name = Epoch"
	descripStr(2)="ArgumentName = Epoch"
	descripStr(3)="Type = CalcObject"

	argStr(7) = descripStr


	ReDim descripStr(4)

	descripStr(0)="ArgumentType = Input"
	descripStr(1)="Name = Range"
	descripStr(2)="ArgumentName = Range"
	descripStr(3)="Type = CalcObject"

	argStr(8) = descripStr



	docking_Update_register = argStr


	'Scenario path
	ScenPath = ""

'MsgBox "About to Create COM object" 

	'Create COM object an instance of FuzzyDocker Class
	Set FuzzyDockingControl= CreateObject("FuzzySpaceDocker.FuzzyDocker")

	'Set Engine Type		
	FuzzyDockingControl.EngineType = 1
	
	'Set frequency for Fuzzy Logic to evaluate
	FuzzyDockingControl.FuzzyEvalFreq = 10

	'Set Pulse Engine Max Thrust
	FuzzyDockingControl.PulseEngineNewtons = 10

'MsgBox "Create COM object complete " 

	
	'Articulation File	
	FuzzyDockingControl.ArticulationFile = true
	FuzzyDockingControl.ArticulationFilePath = ScenPath  & "Chase.sama"


	'Trace File to display Engione Thrust for Fuzzy Logic Output
	FuzzyDockingControl.TraceFile = true
	FuzzyDockingControl.TraceFilePath = ScenPath  & "FuzzyTrace.csv"

	'Record File for playback in Fuzzy Tech
	FuzzyDockingControl.FTFile = true
	FuzzyDockingControl.FTFilePath = ScenPath  & "FuzzyDocking.csv"


'MsgBox "Trace File complete " 

	'Load Fuzzy System
   	If FuzzyDockingControl.LoadFuzzySystem(ScenPath  & "SPACEDOC.ftr", serr) = False Then	
    		MsgBox "Error Loading: " &  ScenPath  & "SPACEDOC.ftr - " & serr
   	End If
	

	'MsgBox "Registration section complete " 

End Function


Function docking_Update_compute( stateData)
	
' 	The following allow inputs and outputs to be called via c structure syntax

	Set docking_Update_Inputs = g_GetPluginArrayInterface("docking_Update_Inputs")
	Set docking_Update_Outputs = g_GetPluginArrayInterface("docking_Update_Outputs")

	Dim X, Y, Z, Xdot, Ydot, Zdot, Range
	Dim Fx, Fy, Fz 
	Dim returnValue(4), mass, a_sub_t, target_thrust, num_engines
	Dim fuzzy_thrust(7)
	Dim serr

'	MsgBox "Compute section " & vbNewline

	Epoch = stateData(docking_Update_Inputs.Epoch)
	X = stateData(docking_Update_Inputs.X)
	Y = stateData(docking_Update_Inputs.Y)
	Z = stateData(docking_Update_Inputs.Z)
	Xdot = stateData(docking_Update_Inputs.Xdot)
	Ydot = stateData(docking_Update_Inputs.Ydot)
	Zdot = stateData(docking_Update_Inputs.Zdot)
	Range = stateData(docking_Update_Inputs.Range)

	' Set values to some random numbers
	fuzzy_thrust(0) = 0  '- X 1
	fuzzy_thrust(1) = 0  '- X 2	
	fuzzy_thrust(2) = 0 'Plus x direction
	fuzzy_thrust(3) = 0  '+ Z 
	fuzzy_thrust(4) = 0  '-Z
	fuzzy_thrust(5) = 0  '+Y
	fuzzy_thrust(6) = 0  '-Y


'--------FUZZY LOGIC DOCKING CONTROL LAWS ---------------------------------------------------------------------------------------------
	'Call Fuzzy Docking Control using X,Y,Z and Xdot, Ydot, Zdot.  Return values for each engine below
	
	'Pass in Time
	FuzzyDockingControl.EpochSecond = Epoch 


	'Z ENGINE 
	'Fuzzy Returns + Towards Axis, - Away from axis
	Fz = FuzzyDockingControl.FuzzyZEngine(X, Z, Zdot,EngineIndex, serr) 
	
	fuzzy_thrust(EngineIndex) = Fz '+Z	

	'Y ENGINE
	'+ Towards Axis, - Away from axis
	Fy = FuzzyDockingControl.FuzzyYEngine(X, Y, Ydot,EngineIndex, serr) 
	
	fuzzy_thrust(EngineIndex) = Fy 	

	'X ENGINE
	Fx = FuzzyDockingControl.FuzzyXEngine(Xdot, X,EngineIndex, serr)

	If EngineIndex = 1 then  'Split for 2 off center X engines
		fuzzy_thrust(0)  = Fx/2
		fuzzy_thrust(1)  = Fx/2	
		fuzzy_thrust(2) = 0
			
	else
		fuzzy_thrust(EngineIndex) = Fx 	
		fuzzy_thrust(0)  = 0
		fuzzy_thrust(1)  = 0

	end if		

	'Check for error condition in fuzzy logic engine
	if serr <> "" then 		
		FuzzyDockingControl.TraceWriteline("Error FuzzyEngine " & serr)
	end if  

	' Set values for each engine

	' Remember that whatever value you come up with for the -X thrust, split it between engines 1 and 2

	' Set values for engine 1, which is on the +X axis but provides thrust in the -x direction.
	' We're assuming that the variable fuzzy_thrust comes from the fuzzy logic stuff.
	thrust(1) = fuzzy_thrust(0)
	ISP(1) = 310
	MassFlowRate(1) = thrust(1)/(ISP(1)*9.80665)

	' Set values for engine 2, which is on the +X axis but provides thrust in the -x direction.
	' We're assuming that the variable fuzzy_thrust comes from the fuzzy logic stuff.
	thrust(2) = fuzzy_thrust(1)
	ISP(2) = 310
	MassFlowRate(2) = thrust(2)/(ISP(2)*9.80665)

	' Set values for engine 3, which provides thrust in the +X direction.
	' We're assuming that the variable fuzzy_thrust comes from the fuzzy logic stuff.
	thrust(3) = fuzzy_thrust(2)
	ISP(3) = 310
	MassFlowRate(3) = thrust(3)/(ISP(3)*9.80665)

	' Set values for engine 4, which provides thrust in the +Z direction.
	' We're assuming that the variable fuzzy_thrust comes from the fuzzy logic stuff.
	thrust(4) = fuzzy_thrust(3)
	ISP(4) = 310
	MassFlowRate(4) = thrust(4)/(ISP(4)*9.80665)

	' Set values for engine 5, which provides thrust in the -Z direction.
	' We're assuming that the variable fuzzy_thrust comes from the fuzzy logic stuff.
	thrust(5) = fuzzy_thrust(4)
	ISP(5) = 310
	MassFlowRate(5) = thrust(5)/(ISP(5)*9.80665)

	' Set values for engine 6, which provides thrust in the +Y direction.
	' We're assuming that the variable fuzzy_thrust comes from the fuzzy logic stuff.
	thrust(6) = fuzzy_thrust(5)
	ISP(6) = 310
	MassFlowRate(6) = thrust(6)/(ISP(6)*9.80665)

	' Set values for engine 7, which provides thrust in the -Y direction.
	' We're assuming that the variable fuzzy_thrust comes from the fuzzy logic stuff.
	thrust(7) = fuzzy_thrust(6)
	ISP(7) = 310
	MassFlowRate(7) = thrust(7)/(ISP(7)*9.80665)
                                                                           
	returnValue(docking_Update_Outputs.status) = "Still Okay"

	docking_Update_compute = returnValue

End Function

